from bge import logic, render
from math import sqrt

from .hex import cube_area_draw, cube_line, Tile, Hex

SIZE = 2.0		#the length of a hex-side

WIDTH = SIZE * 2	#the width of a hex
HORIZ = WIDTH * 3/4	#horizontal distance between columns
HEIGHT = sqrt(3)/2 * WIDTH	#height of a hexagon, distance between hexes on a row

DISPLAY_CUBIC = 1
	    
	
class Board:
	#an array of hex tiles
	def __init__(self,brush,incoming,mouse_over, mouse_click, width,height):
		self.scene = logic.getCurrentScene()	#used for addObject calls
		self.brush = brush							#used as placement origin for addObject calls
		self.incoming = incoming					#general message reciever
		self.mouse_over = mouse_over				#mouse-over sensor
		self.mouse_click = mouse_click				#mouse click sensor
		
		self.cursor = self.scene.objects['cursor']	#cursor object (tracks mouse cursor on board)
		self.target = self.scene.objects['target']	#target object (tracks currently targeted/selected tile)
		self.cursor.removeParent()						#parenting no longer needed
		self.target.removeParent()						#
		
		self.width = width				#width/height of the board in tiles
		self.height = height
		
		self.map = self.build_grid(width, height)
		
		self.cursorHex = None						#Hex coords for cursor position on board
		self.targetHex = None						#Hex coords for target position on board
		
		self.lineHexes = []							#list of cube coords for drawing lines between tiles
		self.lineHexPips = []						#pips associated with lineHexes
		
		self.areaHexes = []							#list of cube coords for drawing areas around a hex
		self.areaHexPips = []						#pips associated with areaHexes
		
		self.drawLine = False						#do we need to redraw the Line?
		self.drawArea = False						#do we need to redraw the Area?
		
		self.areaRange = 0			#placeholder for updating range
		
		
	#use this to access a tile on the map	
	def get_map_tile(self, hex):
		return self.map[hex.q][hex.r]					#refer to an off-set position on the map.
														
	def find_line(self,A,B):
		distance = int(A.get_distance_to(B))
		logic.sendMessage("DISTANCE", str(distance))
		
		self.lineHexes = [c for c in cube_line(A.hex, B.hex)]
		
		# Clear previous line
		for pip in self.lineHexPips:
			pip.endObject()
		self.lineHexPips.clear()
		
		for hex in self.lineHexes:
			tile = self.get_map_tile(hex)
			self.brush.worldPosition = tile.pip.worldPosition
			pip = self.scene.addObject('MARKER',self.brush)
			self.lineHexPips.append(pip)
			
	#finds an area 'r' from hex 'h' and draws pips on the board
	def find_area(self,h,r):
		for pip in self.areaHexPips:
			pip.endObject()		
			
		self.areaHexPips = []
		self.areaHexes = [c for c in cube_area_draw(self.targetHex, r)]
		
		for hex in self.areaHexes:
			try:
				tile = self.get_map_tile(hex)
			except KeyError:
				continue
			
			self.brush.worldPosition = tile.pip.worldPosition
			pip = self.scene.addObject('MARKER', self.brush)
			self.areaHexPips.append(pip)			
			
	#Board main-loop logic
	def main(self):	
		if self.brush['range'] != self.areaRange:
			self.areaRange = self.brush['range']
			self.drawArea = True
			logic.sendMessage("RANGE",str(self.areaRange))
	
		if self.mouse_over.positive:
			hit = self.mouse_over.hitObject
			tile = hit['tile']
			hex = tile.hex
			
			if not self.cursorHex or hex.q != self.cursorHex.q or hex.r != self.cursorHex.r:
				self.cursorHex = hex
				self.drawLine = True
				
			if self.mouse_click.positive:
				self.targetHex = hex
				self.drawArea = True
		else:
			self.cursorHex = None
		
		if self.cursorHex:
			self.cursor.visible = True
			self.cursor.worldPosition = self.get_map_tile(self.cursorHex).pip.worldPosition
		else:
			self.cursor.visible = False
		
		if self.targetHex:
			self.target.visible = True
			self.target.worldPosition = self.get_map_tile(self.targetHex).pip.worldPosition
			if self.drawArea:
				self.find_area(self.targetHex, self.areaRange)
				self.drawArea = False
		else:
			self.target.visible = False
			
		if self.cursorHex and self.targetHex:
			
			B = self.get_map_tile(self.cursorHex)
			A = self.get_map_tile(self.targetHex)
			
			# Find line hexes
			self.find_line(A,B)
			
			# Get raised start and end points
			s, t = A.pip.worldPosition.copy(), B.pip.worldPosition.copy()
			s.z += 0.5
			t.z += 0.5
			
			# Draw over path PIPs
			render.drawLine(s, t, [1,0,0])
	
	#generate a map for the Board
	def build_grid(self, width, height):
		self.brush.worldPosition = [0, 0, 0]
		
		tiles = {}
		
		# Even Q representation
		for q in range(width):
			q_offset = q // 2
			q_tiles = tiles[q] = {}
			
			for r in range(-q_offset, height - q_offset):
				tile = Tile(q, r)
				q_tiles[r] = tile

				self.brush.worldPosition.x = q * HORIZ
				self.brush.worldPosition.y = -(q/2 + r) * HEIGHT
				
				pip = self.scene.addObject('HEX', self.brush)
				
				# Configure pip
				pip['tile'] = tile
				tile.pip = pip
				
				hex = tile.hex
								
				# Choose display type
				if DISPLAY_CUBIC:
					display = self.scene.addObject('cube_display', pip)
					display.setParent(pip)
										
					display.children['a'].text = str(hex.y)
					display.children['b'].text = str(hex.x)
					display.children['c'].text = str(hex.z)
				
				else:
					display = self.scene.addObject('axial_display', pip)
					display.setParent(pip)
										
					display.children['q'].text = str(hex.q)
					display.children['r'].text = str(hex.r)
				
				
		return tiles