def cube_line(a, b, e=1e-6):
	n = int(a.get_distance_to(b))
	if not n:
		return [] 
	
	# Add slight noise to ensure exact center points don't cause issues
	a = Hex(a.q - e, a.r - e)
	b = Hex(b.q - e, b.r - e)
	
	return [a.lerp(b, (1 / n) * i).rounded() for i in range(n + 1)]
	
	
def cube_area_draw(c, r):
	results = []
	
	x, y, z = c.x, c.y, c.z
	
	xmin = x - r
	xmax = x + r
	
	ymin = y - r
	ymax = y + r
	
	zmin = z - r
	zmax = z + r
		
	for x in range(xmin, xmax+1):
		for y in range(max(ymin, -x - zmax), min(ymax, -x - zmin)+1):
			z = -x - y
			results.append(Hex.from_cube(x, y, z))

	return results

			
class Tile:
	"""A game tile object"""

	def __init__(self, q=0, r=0):
		self.hex = Hex(q, r)
			
	def get_distance_to(self, other):
		#find distance between me and another tile
		return self.hex.get_distance_to(other.hex)
	
	def __repr__(self):
		return "Tile({})".format(self.hex)
			
        
class Hex:

	@classmethod
	def from_cube(cls, x, y, z):
		assert abs(y - (-x - z)) < 1e-6, "Invalid cubic coordinate"
		return cls(x, z)

	def __init__(self, q, r):
		self.q = q
		self.r = r
	
	def __repr__(self):
		return "Hex({}, {})".format(self.q, self.r)
	
	@property
	def x(self):
		return self.q
	
	@x.setter
	def x(self, x):
		self.q = x
	
	@property
	def z(self):
		return self.r
	
	@z.setter
	def z(self, z):
		self.r = z
	
	@property
	def y(self):
		return -self.x - self.z
    
	def rounded(self):
		rx = round(self.x)
		ry = round(self.y)
		rz = round(self.z)

		x_diff = abs(rx - self.x)
		y_diff = abs(ry - self.y)
		z_diff = abs(rz - self.z)

		if x_diff > y_diff and x_diff > z_diff:
			rx = -ry - rz
						
		else:
			rz = -rx - ry
			
		return Hex.from_cube(rx, ry, rz)
    
	def lerp(self, other, factor):
		x = self.x + (other.x - self.x) * factor
		y = self.y + (other.y - self.y) * factor
		z = self.z + (other.z - self.z) * factor
		return Hex.from_cube(x, y, z)
    
	def get_distance_to(self, other):
		dx = abs(self.x - other.x)
		dy = abs(self.y - other.y)
		dz = abs(self.z - other.z)
		return max(dx, dy, dz)
		